@@ -17,8 +17,9 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 
 #include "CanvasElement.hxx"
+#include <simgear/canvas/Canvas.hxx>
 #include <simgear/canvas/CanvasEventVisitor.hxx>
-#include <simgear/canvas/MouseEvent.hxx>
+#include <simgear/canvas/events/MouseEvent.hxx>
 #include <simgear/math/SGMisc.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/scene/material/parseBlendFunc.hxx>
@@ -49,15 +50,50 @@ namespace canvas
   {
     public:
 
-      ReferenceFrame    _coord_reference;
-      osg::Matrix       _parent_inverse;
+      ReferenceFrame                _coord_reference;
+      osg::observer_ptr<osg::Node>  _node;
 
-      RelativeScissor():
-        _coord_reference(GLOBAL)
+      RelativeScissor(osg::Node* node = NULL):
+        _coord_reference(GLOBAL),
+        _node(node)
+      {
+        _width = 0;
+        _height = 0;
+      }
+
+      /** Copy constructor using CopyOp to manage deep vs shallow copy. */
+      RelativeScissor( const RelativeScissor& vp,
+                       const osg::CopyOp& copyop = osg::CopyOp::SHALLOW_COPY ):
+        Scissor(vp, copyop),
+        _coord_reference(vp._coord_reference),
+        _node(vp._node)
       {}
 
+      META_StateAttribute(simgear, RelativeScissor, SCISSOR);
+
+      /** Return -1 if *this < *rhs, 0 if *this==*rhs, 1 if *this>*rhs. */
+      virtual int compare(const StateAttribute& sa) const
+      {
+        // check the types are equal and then create the rhs variable
+        // used by the COMPARE_StateAttribute_Parameter macros below.
+        COMPARE_StateAttribute_Types(RelativeScissor,sa)
+
+        // compare each parameter in turn against the rhs.
+        COMPARE_StateAttribute_Parameter(_x)
+        COMPARE_StateAttribute_Parameter(_y)
+        COMPARE_StateAttribute_Parameter(_width)
+        COMPARE_StateAttribute_Parameter(_height)
+        COMPARE_StateAttribute_Parameter(_coord_reference)
+        COMPARE_StateAttribute_Parameter(_node)
+
+        return 0; // passed all the above comparison macros, must be equal.
+      }
+
       virtual void apply(osg::State& state) const
       {
+        if( _width <= 0 || _height <= 0 )
+          return;
+
         const osg::Viewport* vp = state.getCurrentViewport();
         float w2 = 0.5 * vp->width(),
               h2 = 0.5 * vp->height();
@@ -73,17 +109,28 @@ namespace canvas
 
         if( _coord_reference != GLOBAL )
         {
-          model_view.preMult(state.getModelViewMatrix());
+          osg::Node* ref_obj = _node.get();
 
           if( _coord_reference == PARENT )
-            model_view.preMult(_parent_inverse);
+          {
+            if( _node->getNumParents() < 1 )
+            {
+              SG_LOG(SG_GL, SG_WARN, "RelativeScissor: missing parent.");
+              return;
+            }
+
+            ref_obj = _node->getParent(0);
+          }
+
+          osg::MatrixList const& parent_matrices = ref_obj->getWorldMatrices();
+          assert( !parent_matrices.empty() );
+          model_view.preMult(parent_matrices.front());
         }
 
         const osg::Vec2 scale( model_view(0,0), model_view(1,1)),
                         offset(model_view(3,0), model_view(3,1));
 
         // TODO check/warn for rotation?
-
         GLint x = SGMiscf::roundToInt(scale.x() * _x + offset.x()),
               y = SGMiscf::roundToInt(scale.y() * _y + offset.y()),
               w = SGMiscf::roundToInt(std::fabs(scale.x()) * _width),
@@ -96,6 +143,26 @@ namespace canvas
 
         glScissor(x, y, w, h);
       }
+
+      bool contains(const osg::Vec2f& pos) const
+      {
+        return _x <= pos.x() && pos.x() <= _x + _width
+            && _y <= pos.y() && pos.y() <= _y + _height;
+      }
+
+      bool contains( const osg::Vec2f& global_pos,
+                     const osg::Vec2f& parent_pos,
+                     const osg::Vec2f& local_pos ) const
+      {
+        switch( _coord_reference )
+        {
+          case GLOBAL: return contains(global_pos);
+          case PARENT: return contains(parent_pos);
+          case LOCAL:  return contains(local_pos);
+        }
+
+        return false;
+      }
   };
 
   //----------------------------------------------------------------------------
@@ -123,17 +190,6 @@ namespace canvas
     }
   }
 
-  //----------------------------------------------------------------------------
-  void Element::setSelf(const PropertyBasedElementPtr& self)
-  {
-    PropertyBasedElement::setSelf(self);
-
-    _transform->setUserData
-    (
-      new OSGUserData(boost::static_pointer_cast<Element>(self))
-    );
-  }
-
   //----------------------------------------------------------------------------
   void Element::onDestroy()
   {
@@ -146,43 +202,41 @@ namespace canvas
     {
       parent->removeChild(_transform.get());
     }
+
+    // Hide in case someone still holds a reference
+    setVisible(false);
+    removeListener();
+
+    _parent = 0;
+    _transform = 0;
   }
 
   //----------------------------------------------------------------------------
-  ElementWeakPtr Element::getWeakPtr() const
+  ElementPtr Element::getParent() const
   {
-    return boost::static_pointer_cast<Element>(_self.lock());
+    return _parent;
   }
 
   //----------------------------------------------------------------------------
-  ElementPtr Element::getParent()
+  CanvasWeakPtr Element::getCanvas() const
   {
-    return _parent ? _parent->getWeakPtr().lock() : ElementPtr();
+    return _canvas;
   }
 
   //----------------------------------------------------------------------------
   void Element::update(double dt)
   {
-    if( !_transform->getNodeMask() )
-      // Don't do anything if element is hidden
+    if( !isVisible() )
       return;
 
     // Trigger matrix update
     getMatrix();
 
-    if( _attributes_dirty & SCISSOR_COORDS )
-    {
-      if( _scissor && _scissor->_coord_reference != GLOBAL )
-        _scissor->_parent_inverse = _transform->getInverseMatrix();
-
-      _attributes_dirty &= ~SCISSOR_COORDS;
-    }
-
     // Update bounding box on manual update (manual updates pass zero dt)
     if( dt == 0 && _drawable )
       _drawable->getBound();
 
-    if( _attributes_dirty & BLEND_FUNC )
+    if( (_attributes_dirty & BLEND_FUNC) && _transform.valid() )
     {
       parseBlendFunc(
         _transform->getOrCreateStateSet(),
@@ -208,17 +262,7 @@ namespace canvas
       "addEventListener(" << _node->getPath() << ", " << type_str << ")"
     );
 
-    Event::Type type = Event::strToType(type_str);
-    if( type == Event::UNKNOWN )
-    {
-      SG_LOG( SG_GENERAL,
-              SG_WARN,
-              "addEventListener: Unknown event type " << type_str );
-      return false;
-    }
-
-    _listener[ type ].push_back(cb);
-
+    _listener[ Event::getOrRegisterType(type_str) ].push_back(cb);
     return true;
   }
 
@@ -252,30 +296,74 @@ namespace canvas
   }
 
   //----------------------------------------------------------------------------
-  bool Element::handleEvent(canvas::EventPtr event)
+  bool Element::handleEvent(const EventPtr& event)
   {
     ListenerMap::iterator listeners = _listener.find(event->getType());
     if( listeners == _listener.end() )
       return false;
 
     BOOST_FOREACH(EventListener const& listener, listeners->second)
-      listener(event);
+      try
+      {
+        listener(event);
+      }
+      catch( std::exception const& ex )
+      {
+        SG_LOG(
+          SG_GENERAL,
+          SG_WARN,
+          "canvas::Element: event handler error: '" << ex.what() << "'"
+        );
+      }
 
     return true;
   }
 
   //----------------------------------------------------------------------------
-  bool Element::hitBound( const osg::Vec2f& pos,
+  bool Element::dispatchEvent(const EventPtr& event)
+  {
+    EventPropagationPath path;
+    path.push_back( EventTarget(this) );
+
+    for( Element* parent = _parent;
+                  parent != NULL;
+                  parent = parent->_parent )
+      path.push_front( EventTarget(parent) );
+
+    CanvasPtr canvas = _canvas.lock();
+    if( !canvas )
+      return false;
+
+    return canvas->propagateEvent(event, path);
+  }
+
+  //----------------------------------------------------------------------------
+  bool Element::hitBound( const osg::Vec2f& global_pos,
+                          const osg::Vec2f& parent_pos,
                           const osg::Vec2f& local_pos ) const
   {
-    const osg::Vec3f pos3(pos, 0);
+    if( _scissor && !_scissor->contains(global_pos, parent_pos, local_pos) )
+      return false;
+
+    const osg::Vec3f pos3(parent_pos, 0);
 
     // Drawables have a bounding box...
     if( _drawable )
       return _drawable->getBound().contains(osg::Vec3f(local_pos, 0));
-    // ... for other elements, i.e. groups only a bounding sphere is available
+    else if( _transform.valid() )
+      // ... for other elements, i.e. groups only a bounding sphere is available
+      return _transform->getBound().contains(osg::Vec3f(parent_pos, 0));
     else
-      return _transform->getBound().contains(osg::Vec3f(pos, 0));
+      return false;
+  }
+
+
+  //----------------------------------------------------------------------------
+  void Element::setVisible(bool visible)
+  {
+    if( _transform.valid() )
+      // TODO check if we need another nodemask
+      _transform->setNodeMask(visible ? 0xffffffff : 0);
   }
 
   //----------------------------------------------------------------------------
@@ -296,6 +384,18 @@ namespace canvas
     return _transform.get();
   }
 
+  //----------------------------------------------------------------------------
+  osg::Vec2f Element::posToLocal(const osg::Vec2f& pos) const
+  {
+    getMatrix();
+    const osg::Matrix& m = _transform->getInverseMatrix();
+    return osg::Vec2f
+    (
+      m(0, 0) * pos[0] + m(1, 0) * pos[1] + m(3, 0),
+      m(0, 1) * pos[0] + m(1, 1) * pos[1] + m(3, 1)
+    );
+  }
+
   //----------------------------------------------------------------------------
   void Element::childAdded(SGPropertyNode* parent, SGPropertyNode* child)
   {
@@ -380,7 +480,9 @@ namespace canvas
     if( parent == _node )
     {
       const std::string& name = child->getNameString();
-      if( StyleInfo const* style_info = getStyleInfo(name) )
+      if( boost::starts_with(name, "data-") )
+        return;
+      else if( StyleInfo const* style_info = getStyleInfo(name) )
       {
         SGPropertyNode const* style = child;
         if( isStyleEmpty(child) )
@@ -393,13 +495,11 @@ namespace canvas
       }
       else if( name == "update" )
         return update(0);
-      else if( name == "visible" )
-        // TODO check if we need another nodemask
-        return _transform->setNodeMask( child->getBoolValue() ? 0xffffffff : 0 );
       else if( boost::starts_with(name, "blend-") )
         return (void)(_attributes_dirty |= BLEND_FUNC);
     }
-    else if(   parent->getParent() == _node
+    else if(   parent
+            && parent->getParent() == _node
             && parent->getNameString() == NAME_TRANSFORM )
     {
       _attributes_dirty |= TRANSFORM;
@@ -419,9 +519,13 @@ namespace canvas
   //----------------------------------------------------------------------------
   void Element::setClip(const std::string& clip)
   {
+    osg::StateSet* ss = getOrCreateStateSet();
+    if( !ss )
+      return;
+
     if( clip.empty() || clip == "auto" )
     {
-      getOrCreateStateSet()->removeAttribute(osg::StateAttribute::SCISSOR);
+      ss->removeAttribute(osg::StateAttribute::SCISSOR);
       _scissor = 0;
       return;
     }
@@ -468,47 +572,49 @@ namespace canvas
       return;
     }
 
-    _scissor = new RelativeScissor();
+    if( !_scissor )
+      _scissor = new RelativeScissor(_transform.get());
+
     // <top>, <right>, <bottom>, <left>
     _scissor->x() = SGMiscf::roundToInt(values[3]);
     _scissor->y() = SGMiscf::roundToInt(values[0]);
     _scissor->width() = SGMiscf::roundToInt(width);
     _scissor->height() = SGMiscf::roundToInt(height);
 
-    getOrCreateStateSet()->setAttributeAndModes(_scissor);
-
     SGPropertyNode* clip_frame = _node->getChild("clip-frame", 0);
     if( clip_frame )
       valueChanged(clip_frame);
+    else
+      _scissor->_coord_reference = GLOBAL;
+
+    ss->setAttributeAndModes(_scissor);
   }
 
   //----------------------------------------------------------------------------
   void Element::setClipFrame(ReferenceFrame rf)
   {
     if( _scissor )
-    {
       _scissor->_coord_reference = rf;
-      _attributes_dirty |= SCISSOR_COORDS;
-    }
   }
 
   //----------------------------------------------------------------------------
-  void Element::setBoundingBox(const osg::BoundingBox& bb)
+  osg::BoundingBox Element::getBoundingBox() const
   {
-    if( _bounding_box.empty() )
-    {
-      SGPropertyNode* bb_node = _node->getChild("bounding-box", 0, true);
-      _bounding_box.resize(4);
-      _bounding_box[0] = bb_node->getChild("min-x", 0, true);
-      _bounding_box[1] = bb_node->getChild("min-y", 0, true);
-      _bounding_box[2] = bb_node->getChild("max-x", 0, true);
-      _bounding_box[3] = bb_node->getChild("max-y", 0, true);
-    }
+    if( _drawable )
+      return _drawable->getBound();
 
-    _bounding_box[0]->setFloatValue(bb._min.x());
-    _bounding_box[1]->setFloatValue(bb._min.y());
-    _bounding_box[2]->setFloatValue(bb._max.x());
-    _bounding_box[3]->setFloatValue(bb._max.y());
+    osg::BoundingBox bb;
+
+    if( _transform.valid() )
+      bb.expandBy(_transform->getBound());
+
+    return bb;
+  }
+
+  //----------------------------------------------------------------------------
+  osg::BoundingBox Element::getTightBoundingBox() const
+  {
+    return getTransformedBounds(getMatrix());
   }
 
   //----------------------------------------------------------------------------
@@ -528,6 +634,9 @@ namespace canvas
   //----------------------------------------------------------------------------
   osg::Matrix Element::getMatrix() const
   {
+    if( !_transform )
+      return osg::Matrix::identity();
+
     if( !(_attributes_dirty & TRANSFORM) )
       return _transform->getMatrix();
 
@@ -587,7 +696,6 @@ namespace canvas
     }
     _transform->setMatrix(m);
     _attributes_dirty &= ~TRANSFORM;
-    _attributes_dirty |= SCISSOR_COORDS;
 
     return m;
   }
@@ -626,6 +734,8 @@ namespace canvas
                 "PreOrderBin",
                 osg::StateSet::OVERRIDE_RENDERBIN_DETAILS
               );
+
+    _transform->setUserData( new OSGUserData(this) );
   }
 
   //----------------------------------------------------------------------------
@@ -636,6 +746,7 @@ namespace canvas
 
     addStyle("clip", "", &Element::setClip, false);
     addStyle("clip-frame", "", &Element::setClipFrame, false);
+    addStyle("visible", "", &Element::setVisible, false);
   }
 
   //----------------------------------------------------------------------------
@@ -721,8 +832,12 @@ namespace canvas
   //----------------------------------------------------------------------------
   osg::StateSet* Element::getOrCreateStateSet()
   {
-    return _drawable ? _drawable->getOrCreateStateSet()
-                     : _transform->getOrCreateStateSet();
+    if( _drawable.valid() )
+      return _drawable->getOrCreateStateSet();
+    if( _transform.valid() )
+      return _transform->getOrCreateStateSet();
+
+    return 0;
   }
 
   //----------------------------------------------------------------------------
