@@ -40,7 +40,6 @@
 #include <osg/TexEnv>
 #include <osg/TexEnvCombine>
 #include <osg/Texture2D>
-#include <osg/TextureCubeMap>
 #include <osg/TexMat>
 #include <osg/Fog>
 
@@ -60,21 +59,8 @@
 using namespace simgear;
 using namespace osg;
 
-#if defined(__MINGW32__)
-#define isnan(x) _isnan(x)
-#endif
-
-// #if defined (__FreeBSD__)
-// #  if __FreeBSD_version < 500000
-//      extern "C" {
-//        inline int isnan(double r) { return !(r <= 0 || r >= 0); }
-//      }
-// #  endif
-// #endif
-
 static osg::ref_ptr<osg::StateSet> layer_states[SGCloudLayer::SG_MAX_CLOUD_COVERAGES];
 static osg::ref_ptr<osg::StateSet> layer_states2[SGCloudLayer::SG_MAX_CLOUD_COVERAGES];
-static osg::ref_ptr<osg::TextureCubeMap> cubeMap;
 static bool state_initialized = false;
 
 const std::string SGCloudLayer::SG_CLOUD_OVERCAST_STRING = "overcast";
@@ -101,31 +87,8 @@ SGMakeState(const SGPath &path, const char* colorTexture,
     stateSet->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
     stateSet->setAttributeAndModes(attribFactory->getStandardAlphaFunc());
     stateSet->setAttributeAndModes(attribFactory->getStandardBlendFunc());
-
-//     osg::Material* material = new osg::Material;
-//     material->setColorMode(osg::Material::AMBIENT_AND_DIFFUSE);
-//     material->setEmission(osg::Material::FRONT_AND_BACK,
-//                           osg::Vec4(0.05, 0.05, 0.05, 0));
-//     material->setSpecular(osg::Material::FRONT_AND_BACK,
-//                           osg::Vec4(0, 0, 0, 1));
-//     stateSet->setAttribute(material);
-
     stateSet->setMode(GL_FOG, osg::StateAttribute::OFF);
 
-    // OSGFIXME: invented by me ...
-//     stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
-//     stateSet->setMode(GL_LIGHTING, osg::StateAttribute::ON);
-
-//     stateSet->setMode(GL_LIGHT0, osg::StateAttribute::OFF);
-
-    // If the normal texture is given prepare a bumpmapping enabled state
-//     if (normalTexture) {
-//       SGPath normalPath(path);
-//       normalPath.append(normalTexture);
-//       stateSet->setTextureAttribute(2, SGLoadTexture2D(normalPath));
-//       stateSet->setTextureMode(2, GL_TEXTURE_2D, osg::StateAttribute::ON);
-//     }
-
     return stateSet;
 }
 
@@ -194,7 +157,7 @@ SGCloudLayer::SGCloudLayer( const string &tex_path ) :
                                        ->getWhiteTexture(),
                                        osg::StateAttribute::ON);
   rootSet->setDataVariance(osg::Object::DYNAMIC);
-  
+
   // Ensure repeatability of the random seed within 10 minutes,
   // to keep multi-computer systems in sync.
   sg_srandom_time_10();
@@ -319,7 +282,7 @@ SGCloudLayer::getCoverageString( Coverage coverage )
 	}
 }
 
-SGCloudLayer::Coverage 
+SGCloudLayer::Coverage
 SGCloudLayer::getCoverageType( const std::string & coverage )
 {
 	if( SG_CLOUD_OVERCAST_STRING == coverage ) {
@@ -374,153 +337,33 @@ void
 SGCloudLayer::rebuild()
 {
     // Initialize states and sizes if necessary.
-    if ( !state_initialized ) { 
+    if ( !state_initialized ) {
         state_initialized = true;
 
         SG_LOG(SG_ASTRO, SG_INFO, "initializing cloud layers");
 
-        // This bump mapping code was inspired by the tutorial available at 
-        // http://www.paulsprojects.net/tutorials/simplebump/simplebump.html
-        // and a NVidia white paper 
-        //  http://developer.nvidia.com/object/bumpmappingwithregistercombiners.html
-        // The normal map textures were generated by the normal map Gimp plugin :
-        //  http://nifelheim.dyndns.org/~cocidius/normalmap/
-        //
-        cubeMap = new osg::TextureCubeMap;
-        cubeMap->setFilter(osg::Texture::MIN_FILTER, osg::Texture::LINEAR);
-        cubeMap->setFilter(osg::Texture::MAG_FILTER, osg::Texture::LINEAR);
-        cubeMap->setWrap(osg::Texture::WRAP_S, osg::Texture::CLAMP_TO_EDGE);
-        cubeMap->setWrap(osg::Texture::WRAP_T, osg::Texture::CLAMP_TO_EDGE);
-        cubeMap->setWrap(osg::Texture::WRAP_R, osg::Texture::CLAMP_TO_EDGE);
-
-        const int size = 32;
-        const float half_size = 16.0f;
-        const float offset = 0.5f;
-        osg::Vec3 zero_normal(0.5, 0.5, 0.5);
-
-        osg::Image* image = new osg::Image;
-        image->allocateImage(size, size, 1, GL_RGB, GL_UNSIGNED_BYTE);
-        unsigned char *ptr = image->data(0, 0);
-        for (int j = 0; j < size; j++ ) {
-          for (int i = 0; i < size; i++ ) {
-            osg::Vec3 tmp(half_size, -( j + offset - half_size ),
-                          -( i + offset - half_size ) );
-            tmp.normalize();
-            tmp = tmp*0.5 - zero_normal;
-            
-            *ptr++ = (unsigned char)( tmp[ 0 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 1 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 2 ] * 255 );
-          }
-        }
-        cubeMap->setImage(osg::TextureCubeMap::POSITIVE_X, image);
-
-        image = new osg::Image;
-        image->allocateImage(size, size, 1, GL_RGB, GL_UNSIGNED_BYTE);
-        ptr = image->data(0, 0);
-        for (int j = 0; j < size; j++ ) {
-          for (int i = 0; i < size; i++ ) {
-            osg::Vec3 tmp(-half_size, -( j + offset - half_size ),
-                          ( i + offset - half_size ) );
-            tmp.normalize();
-            tmp = tmp*0.5 - zero_normal;
-            
-            *ptr++ = (unsigned char)( tmp[ 0 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 1 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 2 ] * 255 );
-          }
-        }
-        cubeMap->setImage(osg::TextureCubeMap::NEGATIVE_X, image);
-
-        image = new osg::Image;
-        image->allocateImage(size, size, 1, GL_RGB, GL_UNSIGNED_BYTE);
-        ptr = image->data(0, 0);
-        for (int j = 0; j < size; j++ ) {
-          for (int i = 0; i < size; i++ ) {
-            osg::Vec3 tmp(( i + offset - half_size ), half_size,
-                          ( j + offset - half_size ) );
-            tmp.normalize();
-            tmp = tmp*0.5 - zero_normal;
-            
-            *ptr++ = (unsigned char)( tmp[ 0 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 1 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 2 ] * 255 );
-          }
-        }
-        cubeMap->setImage(osg::TextureCubeMap::POSITIVE_Y, image);
-
-        image = new osg::Image;
-        image->allocateImage(size, size, 1, GL_RGB, GL_UNSIGNED_BYTE);
-        ptr = image->data(0, 0);
-        for (int j = 0; j < size; j++ ) {
-          for (int i = 0; i < size; i++ ) {
-            osg::Vec3 tmp(( i + offset - half_size ), -half_size,
-                          -( j + offset - half_size ) );
-            tmp.normalize();
-            tmp = tmp*0.5 - zero_normal;
-
-            *ptr++ = (unsigned char)( tmp[ 0 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 1 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 2 ] * 255 );
-          }
-        }
-        cubeMap->setImage(osg::TextureCubeMap::NEGATIVE_Y, image);
-
-        image = new osg::Image;
-        image->allocateImage(size, size, 1, GL_RGB, GL_UNSIGNED_BYTE);
-        ptr = image->data(0, 0);
-        for (int j = 0; j < size; j++ ) {
-          for (int i = 0; i < size; i++ ) {
-            osg::Vec3 tmp(( i + offset - half_size ),
-                          -( j + offset - half_size ), half_size );
-            tmp.normalize();
-            tmp = tmp*0.5 - zero_normal;
-            
-            *ptr++ = (unsigned char)( tmp[ 0 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 1 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 2 ] * 255 );
-          }
-        }
-        cubeMap->setImage(osg::TextureCubeMap::POSITIVE_Z, image);
-
-        image = new osg::Image;
-        image->allocateImage(size, size, 1, GL_RGB, GL_UNSIGNED_BYTE);
-        ptr = image->data(0, 0);
-        for (int j = 0; j < size; j++ ) {
-          for (int i = 0; i < size; i++ ) {
-            osg::Vec3 tmp(-( i + offset - half_size ),
-                          -( j + offset - half_size ), -half_size );
-            tmp.normalize();
-            tmp = tmp*0.5 - zero_normal;
-            *ptr++ = (unsigned char)( tmp[ 0 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 1 ] * 255 );
-            *ptr++ = (unsigned char)( tmp[ 2 ] * 255 );
-          }
-        }
-        cubeMap->setImage(osg::TextureCubeMap::NEGATIVE_Z, image);
-
         osg::StateSet* state;
         state = SGMakeState(texture_path, "overcast.png", "overcast_n.png");
         layer_states[SG_CLOUD_OVERCAST] = state;
         state = SGMakeState(texture_path, "overcast_top.png", "overcast_top_n.png");
         layer_states2[SG_CLOUD_OVERCAST] = state;
-        
+
         state = SGMakeState(texture_path, "broken.png", "broken_n.png");
         layer_states[SG_CLOUD_BROKEN] = state;
         layer_states2[SG_CLOUD_BROKEN] = state;
-        
+
         state = SGMakeState(texture_path, "scattered.png", "scattered_n.png");
         layer_states[SG_CLOUD_SCATTERED] = state;
         layer_states2[SG_CLOUD_SCATTERED] = state;
-        
+
         state = SGMakeState(texture_path, "few.png", "few_n.png");
         layer_states[SG_CLOUD_FEW] = state;
         layer_states2[SG_CLOUD_FEW] = state;
-        
+
         state = SGMakeState(texture_path, "cirrus.png", "cirrus_n.png");
         layer_states[SG_CLOUD_CIRRUS] = state;
         layer_states2[SG_CLOUD_CIRRUS] = state;
-        
+
         layer_states[SG_CLOUD_CLEAR] = 0;
         layer_states2[SG_CLOUD_CLEAR] = 0;
 #if 1
@@ -545,8 +388,8 @@ SGCloudLayer::rebuild()
     // build the cloud layer
     const float layer_scale = layer_span / scale;
     const float mpi = SG_PI/4;
-    
-    // caclculate the difference between a flat-earth model and 
+
+    // caclculate the difference between a flat-earth model and
     // a round earth model given the span and altutude ASL of
     // the cloud layer. This is the difference in altitude between
     // the top of the inverted bowl and the edge of the bowl.
@@ -555,38 +398,38 @@ SGCloudLayer::rebuild()
     const float layer_angle = 0.5*layer_span / layer_to_core; // The angle is half the span
     const float border_to_core = layer_to_core * cos(layer_angle);
     const float alt_diff = layer_to_core - border_to_core;
-    
+
     for (int i = 0; i < 4; i++) {
       if ( layer[i] != NULL ) {
         layer_transform->removeChild(layer[i].get()); // automatic delete
       }
-      
+
       vl[i] = new osg::Vec3Array;
       cl[i] = new osg::Vec4Array;
       tl[i] = new osg::Vec2Array;
-      
-      
+
+
       osg::Vec3 vertex(layer_span*(i-2)/2, -layer_span,
                        alt_diff * (sin(i*mpi) - 2));
       osg::Vec2 tc(layer_scale * i/4, 0.0f);
       osg::Vec4 color(cloudColors[0], (i == 0) ? 0.0f : 0.15f);
-      
+
       cl[i]->push_back(color);
       vl[i]->push_back(vertex);
       tl[i]->push_back(tc);
-      
+
       for (int j = 0; j < 4; j++) {
         vertex = osg::Vec3(layer_span*(i-1)/2, layer_span*(j-2)/2,
                            alt_diff * (sin((i+1)*mpi) + sin(j*mpi) - 2));
         tc = osg::Vec2(layer_scale * (i+1)/4, layer_scale * j/4);
         color = osg::Vec4(cloudColors[0],
-                          ( (j == 0) || (i == 3)) ?  
+                          ( (j == 0) || (i == 3)) ?
                           ( (j == 0) && (i == 3)) ? 0.0f : 0.15f : 1.0f );
-        
+
         cl[i]->push_back(color);
         vl[i]->push_back(vertex);
         tl[i]->push_back(tc);
-        
+
         vertex = osg::Vec3(layer_span*(i-2)/2, layer_span*(j-1)/2,
                            alt_diff * (sin(i*mpi) + sin((j+1)*mpi) - 2) );
         tc = osg::Vec2(layer_scale * i/4, layer_scale * (j+1)/4 );
@@ -597,28 +440,27 @@ SGCloudLayer::rebuild()
         vl[i]->push_back(vertex);
         tl[i]->push_back(tc);
       }
-      
-      vertex = osg::Vec3(layer_span*(i-1)/2, layer_span, 
+
+      vertex = osg::Vec3(layer_span*(i-1)/2, layer_span,
                          alt_diff * (sin((i+1)*mpi) - 2));
-      
+
       tc = osg::Vec2(layer_scale * (i+1)/4, layer_scale);
-      
+
       color = osg::Vec4(cloudColors[0], (i == 3) ? 0.0f : 0.15f );
-      
+
       cl[i]->push_back( color );
       vl[i]->push_back( vertex );
       tl[i]->push_back( tc );
-      
+
       osg::Geometry* geometry = new osg::Geometry;
       geometry->setUseDisplayList(false);
       geometry->setVertexArray(vl[i].get());
       geometry->setNormalBinding(osg::Geometry::BIND_OFF);
-      geometry->setColorArray(cl[i].get());
-      geometry->setColorBinding(osg::Geometry::BIND_PER_VERTEX);
-      geometry->setTexCoordArray(0, tl[i].get());
+      geometry->setColorArray(cl[i].get(), osg::Array::BIND_PER_VERTEX);
+      geometry->setTexCoordArray(0, tl[i].get(), osg::Array::BIND_PER_VERTEX);
       geometry->addPrimitiveSet(new osg::DrawArrays(GL_TRIANGLE_STRIP, 0, vl[i]->size()));
       layer[i] = new osg::Geode;
-      
+
       std::stringstream sstr;
       sstr << "Cloud Layer (" << i << ")";
       geometry->setName(sstr.str());
@@ -626,7 +468,7 @@ SGCloudLayer::rebuild()
       layer[i]->addDrawable(geometry);
       layer_transform->addChild(layer[i].get());
     }
-    
+
     //OSGFIXME: true
     if ( layer_states[layer_coverage].valid() ) {
       osg::CopyOp copyOp;    // shallow copy
@@ -661,7 +503,7 @@ bool SGCloudLayer::repaint( const SGVec3f& fog_color ) {
 bool SGCloudLayer::reposition( const SGVec3f& p, const SGVec3f& up, double lon, double lat,
         		       double alt, double dt )
 {
-  
+
     if (getCoverage() != SGCloudLayer::SG_CLOUD_CLEAR)
     {
         // combine p and asl (meters) to get translation offset
@@ -684,11 +526,11 @@ bool SGCloudLayer::reposition( const SGVec3f& p, const SGVec3f& up, double lon,
         // Point3D zero_elev = current_view.get_cur_zero_elev();
         T.makeTranslate( asl_offset );
 
-        // printf("  Translated to %.2f %.2f %.2f\n", 
+        // printf("  Translated to %.2f %.2f %.2f\n",
         //        zero_elev.x, zero_elev.y, zero_elev.z );
 
         // Rotate to proper orientation
-        // printf("  lon = %.2f  lat = %.2f\n", 
+        // printf("  lon = %.2f  lat = %.2f\n",
         //        lon * SGD_RADIANS_TO_DEGREES,
         //        lat * SGD_RADIANS_TO_DEGREES);
         LON.makeRotate(lon, osg::Vec3(0, 0, 1));
@@ -716,7 +558,7 @@ bool SGCloudLayer::reposition( const SGVec3f& p, const SGVec3f& up, double lon,
         } else {
           layer_root->setAllChildrenOff();
         }
-            
+
 
         // now calculate update texture coordinates
         SGGeod pos = SGGeod::fromRad(lon, lat);
@@ -725,10 +567,10 @@ bool SGCloudLayer::reposition( const SGVec3f& p, const SGVec3f& up, double lon,
         }
 
         double sp_dist = speed*dt;
-        
-        
+
+
         if ( lon != last_pos.getLongitudeRad() || lat != last_pos.getLatitudeRad() || sp_dist != 0 ) {
-            double course = SGGeodesy::courseDeg(last_pos, pos) * SG_DEGREES_TO_RADIANS, 
+            double course = SGGeodesy::courseDeg(last_pos, pos) * SG_DEGREES_TO_RADIANS,
                 dist = SGGeodesy::distanceM(last_pos, pos);
 
             // if start and dest are too close together,
@@ -736,7 +578,7 @@ bool SGCloudLayer::reposition( const SGVec3f& p, const SGVec3f& up, double lon,
             // this happens, lets just use the last known good course.
             // This is a hack, and it would probably be better to make
             // calc_gc_course_dist() more robust.
-            if ( isnan(course) ) {
+            if ( isNaN(course) ) {
                 course = last_course;
             } else {
                 last_course = course;
@@ -804,7 +646,7 @@ bool SGCloudLayer::reposition( const SGVec3f& p, const SGVec3f& up, double lon,
 }
 
 void SGCloudLayer::set_enable3dClouds(bool enable) {
-     
+
     if (layer3D->isDefined3D() && enable) {
         cloud_root->setChildValue(layer3D->getNode(), true);
         cloud_root->setChildValue(layer_root.get(),   false);
