@@ -41,6 +41,8 @@
 #include <osg/StateSet>
 #include <osg/Switch>
 
+#include <osgUtil/Simplifier>
+
 #include <boost/foreach.hpp>
 
 #include <algorithm>
@@ -70,9 +72,14 @@
 #include "SGDirectionalLightBin.hxx"
 #include "GroundLightManager.hxx"
 
-
 #include "pt_lights.hxx"
 
+#define SG_SIMPLIFIER_RATIO 0.001
+#define SG_SIMPLIFIER_MAX_LENGTH 1000.0
+#define SG_SIMPLIFIER_MAX_ERROR 2000.0
+#define SG_OBJECT_RANGE 9000.0
+#define SG_TILE_RADIUS 14000.0
+
 using namespace simgear;
 
 typedef std::map<std::string,SGTexturedTriangleBin> SGMaterialTriangleMap;
@@ -212,72 +219,107 @@ public:
 
   static void
   addTriangleGeometry(SGTexturedTriangleBin& triangles,
-                      const std::vector<SGVec3d>& vertices,
-                      const std::vector<SGVec3f>& normals,
-                      const std::vector<SGVec2f>& texCoords,
-                      const int_list& tris_v,
-                      const int_list& tris_n,
-                      const int_list& tris_tc,
-                      const SGVec2f& tcScale)
+                      const SGBinObject& obj, unsigned grp,
+                      const SGVec2f& tc0Scale, 
+                      const SGVec2f& tc1Scale)
   {
+    const std::vector<SGVec3d>& vertices(obj.get_wgs84_nodes());
+    const std::vector<SGVec3f>& normals(obj.get_normals());
+    const std::vector<SGVec2f>& texCoords(obj.get_texcoords());
+    const int_list& tris_v(obj.get_tris_v()[grp]);
+    const int_list& tris_n(obj.get_tris_n()[grp]);
+    const tci_list& tris_tc(obj.get_tris_tcs()[grp]);
+    bool  num_norms_is_num_verts = true;  
+    
     if (tris_v.size() != tris_n.size()) {
-      // If the normal indices do not match, they should be inmplicitly
-      // the same than the vertex indices. So just call ourselves again
-      // with the matching index vector.
-      addTriangleGeometry(triangles, vertices, normals, texCoords,
-                          tris_v, tris_v, tris_tc, tcScale);
-      return;
+        // If the normal indices do not match, they should be inmplicitly
+        // the same than the vertex indices. 
+        num_norms_is_num_verts = false;
     }
 
+    if ( !tris_tc[1].empty() ) {
+        triangles.hasSecondaryTexCoord(true);
+    }
+    
     for (unsigned i = 2; i < tris_v.size(); i += 3) {
-      SGVertNormTex v0;
-      v0.vertex = toVec3f(vertices[tris_v[i-2]]);
-      v0.normal = normals[tris_n[i-2]];
-      v0.texCoord = getTexCoord(texCoords, tris_tc, tcScale, i-2);
-      SGVertNormTex v1;
-      v1.vertex = toVec3f(vertices[tris_v[i-1]]);
-      v1.normal = normals[tris_n[i-1]];
-      v1.texCoord = getTexCoord(texCoords, tris_tc, tcScale, i-1);
-      SGVertNormTex v2;
-      v2.vertex = toVec3f(vertices[tris_v[i]]);
-      v2.normal = normals[tris_n[i]];
-      v2.texCoord = getTexCoord(texCoords, tris_tc, tcScale, i);
-      triangles.insert(v0, v1, v2);
+        SGVertNormTex v0;
+        v0.SetVertex( toVec3f(vertices[tris_v[i-2]]) );
+        v0.SetNormal( num_norms_is_num_verts ? normals[tris_n[i-2]] : 
+                                               normals[tris_v[i-2]] );
+        v0.SetTexCoord( 0, getTexCoord(texCoords, tris_tc[0], tc0Scale, i-2) );
+        if (!tris_tc[1].empty()) {
+            v0.SetTexCoord( 1, getTexCoord(texCoords, tris_tc[1], tc1Scale, i-2) );
+        }
+        SGVertNormTex v1;
+        v1.SetVertex( toVec3f(vertices[tris_v[i-1]]) );
+        v1.SetNormal( num_norms_is_num_verts ? normals[tris_n[i-1]] : 
+                                               normals[tris_v[i-1]] );
+        v1.SetTexCoord( 0, getTexCoord(texCoords, tris_tc[0], tc0Scale, i-1) );
+        if (!tris_tc[1].empty()) {
+            v1.SetTexCoord( 1, getTexCoord(texCoords, tris_tc[1], tc1Scale, i-1) );
+        }
+        SGVertNormTex v2;
+        v2.SetVertex( toVec3f(vertices[tris_v[i]]) );
+        v2.SetNormal( num_norms_is_num_verts ? normals[tris_n[i]] : 
+                                               normals[tris_v[i]] );
+        v2.SetTexCoord( 0, getTexCoord(texCoords, tris_tc[0], tc0Scale, i) );
+        if (!tris_tc[1].empty()) {
+            v2.SetTexCoord( 1, getTexCoord(texCoords, tris_tc[1], tc1Scale, i) );
+        }
+        
+        triangles.insert(v0, v1, v2);
     }
   }
 
   static void
   addStripGeometry(SGTexturedTriangleBin& triangles,
-                   const std::vector<SGVec3d>& vertices,
-                   const std::vector<SGVec3f>& normals,
-                   const std::vector<SGVec2f>& texCoords,
-                   const int_list& strips_v,
-                   const int_list& strips_n,
-                   const int_list& strips_tc,
-                   const SGVec2f& tcScale)
+                   const SGBinObject& obj, unsigned grp,
+                   const SGVec2f& tc0Scale, 
+                   const SGVec2f& tc1Scale)
   {
-    if (strips_v.size() != strips_n.size()) {
-      // If the normal indices do not match, they should be inmplicitly
-      // the same than the vertex indices. So just call ourselves again
-      // with the matching index vector.
-      addStripGeometry(triangles, vertices, normals, texCoords,
-                       strips_v, strips_v, strips_tc, tcScale);
-      return;
-    }
-
+      const std::vector<SGVec3d>& vertices(obj.get_wgs84_nodes());
+      const std::vector<SGVec3f>& normals(obj.get_normals());
+      const std::vector<SGVec2f>& texCoords(obj.get_texcoords());
+      const int_list& strips_v(obj.get_strips_v()[grp]);
+      const int_list& strips_n(obj.get_strips_n()[grp]);
+      const tci_list& strips_tc(obj.get_strips_tcs()[grp]);
+      bool  num_norms_is_num_verts = true;  
+      
+      if (strips_v.size() != strips_n.size()) {
+          // If the normal indices do not match, they should be inmplicitly
+          // the same than the vertex indices. 
+          num_norms_is_num_verts = false;
+      }
+      
+      if ( !strips_tc[1].empty() ) {
+          triangles.hasSecondaryTexCoord(true);
+      }
+      
     for (unsigned i = 2; i < strips_v.size(); ++i) {
       SGVertNormTex v0;
-      v0.vertex = toVec3f(vertices[strips_v[i-2]]);
-      v0.normal = normals[strips_n[i-2]];
-      v0.texCoord = getTexCoord(texCoords, strips_tc, tcScale, i-2);
+      v0.SetVertex( toVec3f(vertices[strips_v[i-2]]) );
+      v0.SetNormal( num_norms_is_num_verts ? normals[strips_n[i-2]] : 
+                                             normals[strips_v[i-2]] );
+      v0.SetTexCoord( 0, getTexCoord(texCoords, strips_tc[0], tc0Scale, i-2) );
+      if (!strips_tc[1].empty()) {
+          v0.SetTexCoord( 1, getTexCoord(texCoords, strips_tc[1], tc1Scale, i-2) );
+      }
       SGVertNormTex v1;
-      v1.vertex = toVec3f(vertices[strips_v[i-1]]);
-      v1.normal = normals[strips_n[i-1]];
-      v1.texCoord = getTexCoord(texCoords, strips_tc, tcScale, i-1);
+      v1.SetVertex( toVec3f(vertices[strips_v[i-1]]) );
+      v1.SetNormal( num_norms_is_num_verts ? normals[strips_n[i-1]] : 
+                                             normals[strips_v[i-1]] );
+      v1.SetTexCoord( 0, getTexCoord(texCoords, strips_tc[1], tc0Scale, i-1) );
+      if (!strips_tc[1].empty()) {
+          v1.SetTexCoord( 1, getTexCoord(texCoords, strips_tc[1], tc1Scale, i-1) );
+      }
       SGVertNormTex v2;
-      v2.vertex = toVec3f(vertices[strips_v[i]]);
-      v2.normal = normals[strips_n[i]];
-      v2.texCoord = getTexCoord(texCoords, strips_tc, tcScale, i);
+      v2.SetVertex( toVec3f(vertices[strips_v[i]]) );
+      v2.SetNormal( num_norms_is_num_verts ? normals[strips_n[i]] : 
+                                             normals[strips_v[i]] );
+      v2.SetTexCoord( 0, getTexCoord(texCoords, strips_tc[0], tc0Scale, i) );
+      if (!strips_tc[1].empty()) {
+          v2.SetTexCoord( 1, getTexCoord(texCoords, strips_tc[1], tc1Scale, i) );
+      }
       if (i%2)
         triangles.insert(v1, v0, v2);
       else
@@ -287,36 +329,53 @@ public:
 
   static void
   addFanGeometry(SGTexturedTriangleBin& triangles,
-                 const std::vector<SGVec3d>& vertices,
-                 const std::vector<SGVec3f>& normals,
-                 const std::vector<SGVec2f>& texCoords,
-                 const int_list& fans_v,
-                 const int_list& fans_n,
-                 const int_list& fans_tc,
-                 const SGVec2f& tcScale)
+                 const SGBinObject& obj, unsigned grp,
+                 const SGVec2f& tc0Scale, 
+                 const SGVec2f& tc1Scale)
   {
-    if (fans_v.size() != fans_n.size()) {
-      // If the normal indices do not match, they should be implicitly
-      // the same than the vertex indices. So just call ourselves again
-      // with the matching index vector.
-      addFanGeometry(triangles, vertices, normals, texCoords,
-                     fans_v, fans_v, fans_tc, tcScale);
-      return;
-    }
-
+      const std::vector<SGVec3d>& vertices(obj.get_wgs84_nodes());
+      const std::vector<SGVec3f>& normals(obj.get_normals());
+      const std::vector<SGVec2f>& texCoords(obj.get_texcoords());
+      const int_list& fans_v(obj.get_fans_v()[grp]);
+      const int_list& fans_n(obj.get_fans_n()[grp]);
+      const tci_list& fans_tc(obj.get_fans_tcs()[grp]);
+      bool  num_norms_is_num_verts = true;  
+      
+      if (fans_v.size() != fans_n.size()) {
+          // If the normal indices do not match, they should be inmplicitly
+          // the same than the vertex indices. 
+          num_norms_is_num_verts = false;
+      }
+      
+      if ( !fans_tc[1].empty() ) {
+          triangles.hasSecondaryTexCoord(true);
+      }
+      
     SGVertNormTex v0;
-    v0.vertex = toVec3f(vertices[fans_v[0]]);
-    v0.normal = normals[fans_n[0]];
-    v0.texCoord = getTexCoord(texCoords, fans_tc, tcScale, 0);
+    v0.SetVertex( toVec3f(vertices[fans_v[0]]) );
+    v0.SetNormal( num_norms_is_num_verts ? normals[fans_n[0]] : 
+                                           normals[fans_v[0]] );
+    v0.SetTexCoord( 0, getTexCoord(texCoords, fans_tc[0], tc0Scale, 0) );
+    if (!fans_tc[1].empty()) {
+        v0.SetTexCoord( 1, getTexCoord(texCoords, fans_tc[1], tc1Scale, 0) );
+    }
     SGVertNormTex v1;
-    v1.vertex = toVec3f(vertices[fans_v[1]]);
-    v1.normal = normals[fans_n[1]];
-    v1.texCoord = getTexCoord(texCoords, fans_tc, tcScale, 1);
+    v1.SetVertex( toVec3f(vertices[fans_v[1]]) );
+    v1.SetNormal( num_norms_is_num_verts ? normals[fans_n[1]] : 
+                                           normals[fans_v[1]] );
+    v1.SetTexCoord( 0, getTexCoord(texCoords, fans_tc[0], tc0Scale, 1) );
+    if (!fans_tc[1].empty()) {
+        v1.SetTexCoord( 1, getTexCoord(texCoords, fans_tc[1], tc1Scale, 1) );
+    }
     for (unsigned i = 2; i < fans_v.size(); ++i) {
       SGVertNormTex v2;
-      v2.vertex = toVec3f(vertices[fans_v[i]]);
-      v2.normal = normals[fans_n[i]];
-      v2.texCoord = getTexCoord(texCoords, fans_tc, tcScale, i);
+      v2.SetVertex( toVec3f(vertices[fans_v[i]]) );
+      v2.SetNormal( num_norms_is_num_verts ? normals[fans_n[i]] : 
+                                             normals[fans_v[i]] );
+      v2.SetTexCoord( 0, getTexCoord(texCoords, fans_tc[0], tc0Scale, i) );
+      if (!fans_tc[1].empty()) {
+          v2.SetTexCoord( 1, getTexCoord(texCoords, fans_tc[1], tc1Scale, i) );
+      }
       triangles.insert(v0, v1, v2);
       v1 = v2;
     }
@@ -337,7 +396,7 @@ public:
   insertSurfaceGeometry(const SGBinObject& obj, SGMaterialLib* matlib)
   {
     if (obj.get_tris_n().size() < obj.get_tris_v().size() ||
-        obj.get_tris_tc().size() < obj.get_tris_v().size()) {
+        obj.get_tris_tcs().size() < obj.get_tris_v().size()) {
       SG_LOG(SG_TERRAIN, SG_ALERT,
              "Group list sizes for triangles do not match!");
       return false;
@@ -345,77 +404,78 @@ public:
 
     for (unsigned grp = 0; grp < obj.get_tris_v().size(); ++grp) {
       std::string materialName = obj.get_tri_materials()[grp];
-      SGVec2f tcScale = getTexCoordScale(materialName, matlib);
+      SGVec2f tc0Scale = getTexCoordScale(materialName, matlib);
+      SGVec2f tc1Scale(1.0, 1.0);
       addTriangleGeometry(materialTriangleMap[materialName],
-                          obj.get_wgs84_nodes(), obj.get_normals(),
-                          obj.get_texcoords(), obj.get_tris_v()[grp],
-                          obj.get_tris_n()[grp], obj.get_tris_tc()[grp],
-                          tcScale);
+                          obj, grp, tc0Scale, tc1Scale );
     }
 
     if (obj.get_strips_n().size() < obj.get_strips_v().size() ||
-        obj.get_strips_tc().size() < obj.get_strips_v().size()) {
+        obj.get_strips_tcs().size() < obj.get_strips_v().size()) {
       SG_LOG(SG_TERRAIN, SG_ALERT,
              "Group list sizes for strips do not match!");
       return false;
     }
     for (unsigned grp = 0; grp < obj.get_strips_v().size(); ++grp) {
       std::string materialName = obj.get_strip_materials()[grp];
-      SGVec2f tcScale = getTexCoordScale(materialName, matlib);
+      SGVec2f tc0Scale = getTexCoordScale(materialName, matlib);
+      SGVec2f tc1Scale(1.0, 1.0);
       addStripGeometry(materialTriangleMap[materialName],
-                       obj.get_wgs84_nodes(), obj.get_normals(),
-                       obj.get_texcoords(), obj.get_strips_v()[grp],
-                       obj.get_strips_n()[grp], obj.get_strips_tc()[grp],
-                       tcScale);
+                          obj, grp, tc0Scale, tc1Scale);
     }
 
     if (obj.get_fans_n().size() < obj.get_fans_v().size() ||
-        obj.get_fans_tc().size() < obj.get_fans_v().size()) {
+        obj.get_fans_tcs().size() < obj.get_fans_v().size()) {
       SG_LOG(SG_TERRAIN, SG_ALERT,
              "Group list sizes for fans do not match!");
       return false;
     }
     for (unsigned grp = 0; grp < obj.get_fans_v().size(); ++grp) {
       std::string materialName = obj.get_fan_materials()[grp];
-      SGVec2f tcScale = getTexCoordScale(materialName, matlib);
+      SGVec2f tc0Scale = getTexCoordScale(materialName, matlib);
+      SGVec2f tc1Scale(1.0, 1.0);
       addFanGeometry(materialTriangleMap[materialName],
-                     obj.get_wgs84_nodes(), obj.get_normals(),
-                     obj.get_texcoords(), obj.get_fans_v()[grp],
-                     obj.get_fans_n()[grp], obj.get_fans_tc()[grp],
-                     tcScale);
+                       obj, grp, tc0Scale, tc1Scale );
     }
     return true;
   }
 
-  osg::Node* getSurfaceGeometry(SGMaterialLib* matlib) const
+  osg::Node* getSurfaceGeometry(SGMaterialLib* matlib, bool useVBOs) const
   {
     if (materialTriangleMap.empty())
       return 0;
 
-    EffectGeode* eg = 0;
-    osg::Group* group = (materialTriangleMap.size() > 1 ? new osg::Group : 0);
+    EffectGeode* eg = NULL;
+    osg::Group* group = (materialTriangleMap.size() > 1 ? new osg::Group : NULL);
+    if (group) {
+        group->setName("surfaceGeometryGroup");
+    }
+    
     //osg::Geode* geode = new osg::Geode;
     SGMaterialTriangleMap::const_iterator i;
     for (i = materialTriangleMap.begin(); i != materialTriangleMap.end(); ++i) {
-      osg::Geometry* geometry = i->second.buildGeometry();
-      SGMaterial *mat = 0;
-      if (matlib)
+      osg::Geometry* geometry = i->second.buildGeometry(useVBOs);
+      SGMaterial *mat = NULL;
+      if (matlib) {
         mat = matlib->findCached(i->first);
+      }
       eg = new EffectGeode;
       eg->setName("EffectGeode");
-      if (mat)
+      if (mat) {
         eg->setEffect(mat->get_effect(i->second));
+      }
       eg->addDrawable(geometry);
       eg->runGenerators(geometry);  // Generate extra data needed by effect
       if (group) {
-        group->setName("surfaceGeometryGroup");
         group->addChild(eg);
       }
     }
-    if (group)
+    
+    if (group) {
         return group;
-    else
+    } else {
         return eg;
+    }
   }
 
   void computeRandomSurfaceLights(SGMaterialLib* matlib)
@@ -473,7 +533,8 @@ public:
     SGMaterialLib* matlib,
     float building_density,
     bool use_random_objects,
-    bool use_random_buildings)
+    bool use_random_buildings,
+    bool useVBOs)
   {
     SGMaterialTriangleMap::iterator i;
 
@@ -501,7 +562,7 @@ public:
       SGBuildingBin* bin = NULL;
 
       if (building_coverage > 0) {
-        bin = new SGBuildingBin(mat);
+        bin = new SGBuildingBin(mat, useVBOs);
         randomBuildings.push_back(bin);
       }
 
@@ -514,12 +575,12 @@ public:
       for (unsigned i = 0; i < num; ++i) {
         SGTexturedTriangleBin::triangle_ref triangleRef = triangleBin.getTriangleRef(i);
 
-        SGVec3f vorigin = triangleBin.getVertex(triangleRef[0]).vertex;
-        SGVec3f v0 = triangleBin.getVertex(triangleRef[1]).vertex - vorigin;
-        SGVec3f v1 = triangleBin.getVertex(triangleRef[2]).vertex - vorigin;
-        SGVec2f torigin = triangleBin.getVertex(triangleRef[0]).texCoord;
-        SGVec2f t0 = triangleBin.getVertex(triangleRef[1]).texCoord - torigin;
-        SGVec2f t1 = triangleBin.getVertex(triangleRef[2]).texCoord - torigin;
+        SGVec3f vorigin = triangleBin.getVertex(triangleRef[0]).GetVertex();
+        SGVec3f v0 = triangleBin.getVertex(triangleRef[1]).GetVertex() - vorigin;
+        SGVec3f v1 = triangleBin.getVertex(triangleRef[2]).GetVertex() - vorigin;
+        SGVec2f torigin = triangleBin.getVertex(triangleRef[0]).GetTexCoord(0);
+        SGVec2f t0 = triangleBin.getVertex(triangleRef[1]).GetTexCoord(0) - torigin;
+        SGVec2f t1 = triangleBin.getVertex(triangleRef[2]).GetTexCoord(0) - torigin;
         SGVec3f normal = cross(v0, v1);
 
         // Ensure the slope isn't too steep by checking the
@@ -891,6 +952,10 @@ public:
         group->setName("Random Object and Lighting Group");
         group->setDataVariance(osg::Object::STATIC);
 
+        osg::Node* node = loadTerrain();
+        if (node)
+          group->addChild(node);
+
         osg::LOD* lightLOD = generateLightingTileObjects();
         if (lightLOD)
           group->addChild(lightLOD);
@@ -902,6 +967,67 @@ public:
         return group.release();
     }
 
+    // Load terrain if required
+    osg::Node* loadTerrain()
+    {
+      if (! _loadterrain)
+        return NULL;
+
+      SGBinObject tile;
+      if (!tile.read_bin(_path))
+        return NULL;
+
+      SGMaterialLibPtr matlib;
+      bool useVBOs = false;
+      bool simplifyNear    = false;
+      double ratio       = SG_SIMPLIFIER_RATIO;
+      double maxLength   = SG_SIMPLIFIER_MAX_LENGTH;
+      double maxError    = SG_SIMPLIFIER_MAX_ERROR;
+
+      if (_options) {
+        matlib = _options->getMaterialLib();
+        useVBOs = (_options->getPluginStringData("SimGear::USE_VBOS") == "ON");
+        SGPropertyNode* propertyNode = _options->getPropertyNode().get();
+        simplifyNear = propertyNode->getBoolValue("/sim/rendering/terrain/simplifier/enabled-near", simplifyNear);
+        ratio = propertyNode->getDoubleValue("/sim/rendering/terrain/simplifier/ratio", ratio);
+        maxLength = propertyNode->getDoubleValue("/sim/rendering/terrain/simplifier/max-length", maxLength);
+        maxError = propertyNode->getDoubleValue("/sim/rendering/terrain/simplifier/max-error", maxError);
+      }
+
+      // PSADRO TODO : we can do this in terragear 
+      // - why not add a bitmask of flags to the btg so we can precompute this?
+      // and only do it if it hasn't been done already
+      SGVec3d center = tile.get_gbs_center();
+      SGGeod geodPos = SGGeod::fromCart(center);
+      SGQuatd hlOr = SGQuatd::fromLonLat(geodPos)*SGQuatd::fromEulerDeg(0, 0, 180);
+
+      // rotate the tiles so that the bounding boxes get nearly axis aligned.
+      // this will help the collision tree's bounding boxes a bit ...
+      std::vector<SGVec3d> nodes = tile.get_wgs84_nodes();
+      for (unsigned i = 0; i < nodes.size(); ++i)
+        nodes[i] = hlOr.transform(nodes[i]);
+      tile.set_wgs84_nodes(nodes);
+
+      SGQuatf hlOrf(hlOr[0], hlOr[1], hlOr[2], hlOr[3]);
+      std::vector<SGVec3f> normals = tile.get_normals();
+      for (unsigned i = 0; i < normals.size(); ++i)
+        normals[i] = hlOrf.transform(normals[i]);
+      tile.set_normals(normals);
+
+      osg::ref_ptr<SGTileGeometryBin> tileGeometryBin = new SGTileGeometryBin;
+
+      if (!tileGeometryBin->insertBinObj(tile, matlib))
+        return NULL;
+
+      osg::Node* node = tileGeometryBin->getSurfaceGeometry(matlib, useVBOs);
+      if (node && simplifyNear) {
+        osgUtil::Simplifier simplifier(ratio, maxError, maxLength);
+        node->accept(simplifier);
+      }
+
+      return node;
+    }
+
     // Generate all the lighting objects for the tile.
     osg::LOD* generateLightingTileObjects()
     {
@@ -950,7 +1076,7 @@ public:
       if (!_tileGeometryBin->vasiLights.empty()) {
         EffectGeode* vasiGeode = new EffectGeode;
         Effect* vasiEffect
-            = getLightEffect(24, osg::Vec3(1, 0.0001, 0.000001), 1, 24, true);
+            = getLightEffect(24, osg::Vec3(1, 0.0001, 0.000001), 1, 24, true, _options);
         vasiGeode->setEffect(vasiEffect);
         SGVec4f red(1, 0, 0, 1);
         SGMaterial* mat = 0;
@@ -979,7 +1105,7 @@ public:
           || !_tileGeometryBin->reilLights.empty()
           || !_tileGeometryBin->odalLights.empty()
           || _tileGeometryBin->taxiLights.getNumLights() > 0)
-          runwayEffect = getLightEffect(16, osg::Vec3(1, 0.001, 0.0002), 1, 16, true);
+          runwayEffect = getLightEffect(16, osg::Vec3(1, 0.001, 0.0002), 1, 16, true, _options);
       if (_tileGeometryBin->runwayLights.getNumLights() > 0
           || !_tileGeometryBin->rabitLights.empty()
           || !_tileGeometryBin->reilLights.empty()
@@ -998,24 +1124,24 @@ public:
         SGDirectionalLightListBin::const_iterator i;
         for (i = _tileGeometryBin->rabitLights.begin();
              i != _tileGeometryBin->rabitLights.end(); ++i) {
-          rwyLights->addChild(SGLightFactory::getSequenced(*i));
+          rwyLights->addChild(SGLightFactory::getSequenced(*i, _options));
         }
         for (i = _tileGeometryBin->reilLights.begin();
              i != _tileGeometryBin->reilLights.end(); ++i) {
-          rwyLights->addChild(SGLightFactory::getSequenced(*i));
+          rwyLights->addChild(SGLightFactory::getSequenced(*i, _options));
         }
         for (i = _tileGeometryBin->holdshortLights.begin();
              i != _tileGeometryBin->holdshortLights.end(); ++i) {
-          rwyLights->addChild(SGLightFactory::getHoldShort(*i));
+          rwyLights->addChild(SGLightFactory::getHoldShort(*i, _options));
         }
         for (i = _tileGeometryBin->guardLights.begin();
              i != _tileGeometryBin->guardLights.end(); ++i) {
-          rwyLights->addChild(SGLightFactory::getGuard(*i));
+          rwyLights->addChild(SGLightFactory::getGuard(*i, _options));
         }
         SGLightListBin::const_iterator j;
         for (j = _tileGeometryBin->odalLights.begin();
              j != _tileGeometryBin->odalLights.end(); ++j) {
-          rwyLights->addChild(SGLightFactory::getOdal(*j));
+          rwyLights->addChild(SGLightFactory::getOdal(*j, _options));
         }
         lightGroup->addChild(rwyLights);
       }
@@ -1052,7 +1178,8 @@ public:
       bool use_random_buildings = false;
       float vegetation_density = 1.0f;
       float building_density = 1.0f;
-
+      bool useVBOs = false;
+      
       osg::ref_ptr<osg::Group> randomObjects;
       osg::ref_ptr<osg::Group> forestNode;
       osg::ref_ptr<osg::Group> buildingNode;
@@ -1077,6 +1204,8 @@ public:
                 = propertyNode->getFloatValue("/sim/rendering/building-density",
                                               building_density);
         }
+        
+        useVBOs = (_options->getPluginStringData("SimGear::USE_VBOS") == "ON");
       }
 
 
@@ -1085,7 +1214,8 @@ public:
         _tileGeometryBin->computeRandomObjectsAndBuildings(matlib,
                                                          building_density,
                                                          use_random_objects,
-                                                         use_random_buildings);
+                                                         use_random_buildings,
+                                                         useVBOs);
       }
 
 
@@ -1172,6 +1302,8 @@ public:
     /// The original options to use for this bunch of models
     osg::ref_ptr<SGReaderWriterOptions> _options;
     osg::ref_ptr<SGTileGeometryBin> _tileGeometryBin;
+    string _path;
+    bool _loadterrain;
 };
 
 osg::Node*
@@ -1182,9 +1314,27 @@ SGLoadBTG(const std::string& path, const simgear::SGReaderWriterOptions* options
       return NULL;
 
     SGMaterialLibPtr matlib;
+    bool useVBOs = false;
+    bool simplifyDistant = false;
+    bool simplifyNear    = false;
+    double ratio       = SG_SIMPLIFIER_RATIO;
+    double maxLength   = SG_SIMPLIFIER_MAX_LENGTH;
+    double maxError    = SG_SIMPLIFIER_MAX_ERROR;
+    double object_range = SG_OBJECT_RANGE;
 
     if (options) {
       matlib = options->getMaterialLib();
+      useVBOs = (options->getPluginStringData("SimGear::USE_VBOS") == "ON");
+      SGPropertyNode* propertyNode = options->getPropertyNode().get();
+
+      // We control whether we simplify the nearby terrain and distant terrain separatey.
+      // However, we don't allow only simplifying the near terrain!
+      simplifyNear = propertyNode->getBoolValue("/sim/rendering/terrain/simplifier/enabled-near", simplifyNear);
+      simplifyDistant = simplifyNear || propertyNode->getBoolValue("/sim/rendering/terrain/simplifier/enabled-far", simplifyDistant);
+      ratio = propertyNode->getDoubleValue("/sim/rendering/terrain/simplifier/ratio", ratio);
+      maxLength = propertyNode->getDoubleValue("/sim/rendering/terrain/simplifier/max-length", maxLength);
+      maxError = propertyNode->getDoubleValue("/sim/rendering/terrain/simplifier/max-error", maxError);
+			object_range = propertyNode->getDoubleValue("/sim/rendering/static-lod/rough", object_range);
     }
 
     SGVec3d center = tile.get_gbs_center();
@@ -1209,19 +1359,18 @@ SGLoadBTG(const std::string& path, const simgear::SGReaderWriterOptions* options
     if (!tileGeometryBin->insertBinObj(tile, matlib))
       return NULL;
 
-    osg::Group* terrainGroup = new osg::Group;
-    terrainGroup->setName("BTGTerrainGroup");
 
-    osg::Node* node = tileGeometryBin->getSurfaceGeometry(matlib);
-    if (node)
-      terrainGroup->addChild(node);
+    osg::Node* node = tileGeometryBin->getSurfaceGeometry(matlib, useVBOs);
+    if (node && simplifyDistant) {
+      osgUtil::Simplifier simplifier(ratio, maxError, maxLength);
+      node->accept(simplifier);
+    }
 
     // The toplevel transform for that tile.
     osg::MatrixTransform* transform = new osg::MatrixTransform;
     transform->setName(path);
     transform->setMatrix(osg::Matrix::rotate(toOsg(hlOr))*
                          osg::Matrix::translate(toOsg(center)));
-    transform->addChild(terrainGroup);
 
     // PagedLOD for the random objects so we don't need to generate
     // them all on tile loading.
@@ -1229,17 +1378,34 @@ SGLoadBTG(const std::string& path, const simgear::SGReaderWriterOptions* options
     pagedLOD->setCenterMode(osg::PagedLOD::USE_BOUNDING_SPHERE_CENTER);
     pagedLOD->setName("pagedObjectLOD");
 
+    if (node) {
+      if (simplifyNear == simplifyDistant) {
+        // Same terrain type is used for both near and far distances,
+        // so add it to the main group.
+        osg::Group* terrainGroup = new osg::Group;
+        terrainGroup->setName("BTGTerrainGroup");
+        terrainGroup->addChild(node);
+        transform->addChild(terrainGroup);
+      } else if (simplifyDistant) {
+        // Simplified terrain is only used in the distance, the
+        // call-back below will re-generate the closer version
+        pagedLOD->addChild(node, object_range + SG_TILE_RADIUS, FLT_MAX);
+      }
+    }
+
     // we just need to know about the read file callback that itself holds the data
     osg::ref_ptr<RandomObjectCallback> randomObjectCallback = new RandomObjectCallback;
     randomObjectCallback->_options = SGReaderWriterOptions::copyOrCreate(options);
     randomObjectCallback->_tileGeometryBin = tileGeometryBin;
+    randomObjectCallback->_path = std::string(path);
+    randomObjectCallback->_loadterrain = ! (simplifyNear == simplifyDistant);
 
     osg::ref_ptr<osgDB::Options> callbackOptions = new osgDB::Options;
     callbackOptions->setReadFileCallback(randomObjectCallback.get());
     pagedLOD->setDatabaseOptions(callbackOptions.get());
 
     pagedLOD->setFileName(pagedLOD->getNumChildren(), "Dummy name - use the stored data in the read file callback");
-    pagedLOD->setRange(pagedLOD->getNumChildren(), 0, 35000);
+    pagedLOD->setRange(pagedLOD->getNumChildren(), 0, object_range + SG_TILE_RADIUS);
     transform->addChild(pagedLOD);
     transform->setNodeMask( ~simgear::MODELLIGHT_BIT );
 
